////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2005, Massachusetts Institute of Technology              //
// All rights reserved.                                                   //
//                                                                        //
// Redistribution and use in source and binary forms, with or without     //
// modification, are permitted provided that the following conditions are //
// met:                                                                   //
//                                                                        //
//     * Redistributions of source code must retain the above copyright   //
//       notice, this list of conditions and the following disclaimer.    //
//                                                                        //
//     * Redistributions in binary form must reproduce the above          //
//       copyright notice, this list of conditions and the following      //
//       disclaimer in the documentation and/or other materials provided  //
//       with the distribution.                                           //
//                                                                        //
//     * Neither the name of the Massachusetts Institute of Technology    //
//       nor the names of its contributors may be used to endorse or      //
//       promote products derived from this software without specific     //
//       prior written permission.                                        //
//                                                                        //
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS    //
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT      //
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR  //
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT   //
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,  //
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT       //
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,  //
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY  //
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT    //
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE  //
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.   //
////////////////////////////////////////////////////////////////////////////

#ifndef __FSTDynamicClass_h__
#define __FSTDynamicClass_h__

#include "fst/FST.h"

#undef _L

class FSTDynamicClass {

 public:
    FSTDynamicClass(const std::string& dict_file, const std::string& l2s_file,
                    const std::string& cp_file, const std::string& nickname_file);
    ~FSTDynamicClass();
    void clear(FSTGeneric fst, const std::string& class_name);
    void null(FSTGeneric fst, const std::string& class_name);
    void update(FSTGeneric fst, bool replace, const std::string& class_name,
                int n_list, const char** canonical_list, bool spelling);
    void replace(FSTGeneric fst, const std::string& class_name, const FSTGeneric& contents, bool no_optimize=false);
    FSTGeneric clear(const std::string& class_name);
    FSTGeneric null(const std::string& class_name);
    FSTGeneric update(bool replace, const std::string& class_name,
                      const std::list<std::string>& canonical_list, bool spelling);
    FSTGeneric update(bool replace, const std::string& class_name,
                      int n_list, const char** canonical_list, bool spelling);
    FSTGeneric replace(const std::string& class_name, const FSTGeneric& contents, bool no_optimize=false);

    const fst::set<int>& input_alphabet() const;

    FSTGeneric build_static(const FSTGeneric& G);
    FSTGeneric build_static(const FSTGeneric& G, std::string& errorMsg);

 protected:
    typedef fst::set<int> IntSet;

    struct Class {
        FSTGeneric fst;
        IntSet canonical;
    };

    typedef fst::map<int, Class> IntClassMap;

    FSTGeneric _L;
    FSTGeneric _l2s_fst;
    FSTGeneric _CP;
    FSTGeneric _nickname_fst;
    FSTGeneric _nickname_spelling_fst;
    IntClassMap _class_map;

    int _pound;
    int _iwt;
    int _dollar;

    void _compute_L_star(const FSTGeneric& L_in);
    void _merge(const FSTGeneric& L_new);
    FSTGeneric _construct(const FSTGeneric& x, const std::string& class_name,
                          const FSTGeneric& merge_fst, bool no_optimize);
    void _expand_L(const FSTGeneric& G, std::string& errorMsg);
    void _expand_L(const FSTGeneric& G);

};

#endif //  __FSTDynamicClass_h__
