////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2005, Massachusetts Institute of Technology              //
// All rights reserved.                                                   //
//                                                                        //
// Redistribution and use in source and binary forms, with or without     //
// modification, are permitted provided that the following conditions are //
// met:                                                                   //
//                                                                        //
//     * Redistributions of source code must retain the above copyright   //
//       notice, this list of conditions and the following disclaimer.    //
//                                                                        //
//     * Redistributions in binary form must reproduce the above          //
//       copyright notice, this list of conditions and the following      //
//       disclaimer in the documentation and/or other materials provided  //
//       with the distribution.                                           //
//                                                                        //
//     * Neither the name of the Massachusetts Institute of Technology    //
//       nor the names of its contributors may be used to endorse or      //
//       promote products derived from this software without specific     //
//       prior written permission.                                        //
//                                                                        //
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS    //
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT      //
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR  //
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT   //
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,  //
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT       //
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,  //
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY  //
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT    //
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE  //
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.   //
////////////////////////////////////////////////////////////////////////////

#ifndef __FSTSTL_h__
#define __FSTSTL_h__

#ifdef __GNUC__
#  ifndef __GNUC_PREREQ
#    define __GNUC_PREREQ(maj,min) (__GNUC__ > maj || __GNUC__ == maj && __GNUC_MINOR__ >= min)
#  endif
#endif

#ifdef _MSC_VER
#  ifndef for
#    define for if(false); else for
#  endif
#  pragma warning(disable: 4786)
#endif

#include <algorithm>
#include <functional>
#include <iterator>
#include <memory>
#include <numeric>
#if defined(__GNUC__) && __GNUC__ < 3
#  define queue stl_queue // avoid conflict with Solaris <sys/stream.h>
#  include <queue>
#  undef queue
#else
#  include <queue>
#endif
#include <stack>
#include <utility>

#include "fst/deque"
#include "fst/hash_map"
#include "fst/hash_set"
#include "fst/list"
#include "fst/map"
#include "fst/set"
#include "fst/slist"
#include "fst/vector"

// ptr_less<T> is like STL's less<T>, except it dereferences the pointers before
// comparing.
//
template<class T>
struct ptr_less : std::binary_function<T, T, bool> {
    bool operator()(const T& x, const T& y) const {
        return *x < *y;
    }
};

// Convenient for clearing out a container.
//
template<class T>
inline void clear(T& x)
{
    x.erase(x.begin(), x.end());
}

namespace fst {

    template <class S, class I>
    static inline void set_insert(S& s, I begin, I end)
    {
#ifdef _MSC_VER
        for (I i = begin; i != end; ++i)
            s.insert(*i);
#else
        s.insert(begin, end);
#endif
    }

    template <class V, class I>
    static inline void vector_insert(V& v, I begin, I end)
    {
#ifdef _MSC_VER
        for (I i = begin; i != end; ++i)
            v.push_back(*i);
#else
        v.insert(v.end(), begin, end);
#endif
    }
}

#endif // __FSTSTL_h__

// for Emacs...
// Local Variables:
// mode: C++
// comment-column: 48
// fill-column: 80
// indent-tabs-mode: nil
// c-basic-offset: 4
// End:
