////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2005, Massachusetts Institute of Technology              //
// All rights reserved.                                                   //
//                                                                        //
// Redistribution and use in source and binary forms, with or without     //
// modification, are permitted provided that the following conditions are //
// met:                                                                   //
//                                                                        //
//     * Redistributions of source code must retain the above copyright   //
//       notice, this list of conditions and the following disclaimer.    //
//                                                                        //
//     * Redistributions in binary form must reproduce the above          //
//       copyright notice, this list of conditions and the following      //
//       disclaimer in the documentation and/or other materials provided  //
//       with the distribution.                                           //
//                                                                        //
//     * Neither the name of the Massachusetts Institute of Technology    //
//       nor the names of its contributors may be used to endorse or      //
//       promote products derived from this software without specific     //
//       prior written permission.                                        //
//                                                                        //
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS    //
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT      //
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR  //
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT   //
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,  //
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT       //
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,  //
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY  //
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT    //
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE  //
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.   //
////////////////////////////////////////////////////////////////////////////

#ifndef _FSTStringMap_h_
#define _FSTStringMap_h_

#include "fst/hash_map"
#ifndef NO_HASH_MAP
#  define FSTSTRINGMAP_USE_HASH
#endif

#include "fst/FSTSTL.h"
#include <string>

class FSTStringMap {

public:

    FSTStringMap();
    ~FSTStringMap();
    int operator[](const std::string& s);
    int operator[](const char *s);
    const char *operator[](int i) const;
    int size() const;

protected:

#ifdef FSTSTRINGMAP_USE_HASH

    struct StringHash {
        size_t operator()(const std::string& x) const {
            return HASH_NAMESPACE::hash<const char*>()(x.c_str());
        }
    };
    typedef fst::hash_map<std::string, int, StringHash> _Map;

#else

    typedef std::map<std::string, int> _Map;

#endif

    typedef std::vector<std::string> _Vector;
    _Map fmap;                    // forward map
    _Vector rmap;                 // reverse map
    int n;
};

#endif // _FSTStringMap_h_

// for Emacs...
// Local Variables:
// mode: C++
// comment-column: 50
// fill-column: 110
// c-basic-offset: 4
// End:
