////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2007, Massachusetts Institute of Technology              //
// All rights reserved.                                                   //
//                                                                        //
// Redistribution and use in source and binary forms, with or without     //
// modification, are permitted provided that the following conditions are //
// met:                                                                   //
//                                                                        //
//     * Redistributions of source code must retain the above copyright   //
//       notice, this list of conditions and the following disclaimer.    //
//                                                                        //
//     * Redistributions in binary form must reproduce the above          //
//       copyright notice, this list of conditions and the following      //
//       disclaimer in the documentation and/or other materials provided  //
//       with the distribution.                                           //
//                                                                        //
//     * Neither the name of the Massachusetts Institute of Technology    //
//       nor the names of its contributors may be used to endorse or      //
//       promote products derived from this software without specific     //
//       prior written permission.                                        //
//                                                                        //
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS    //
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT      //
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR  //
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT   //
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,  //
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT       //
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,  //
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY  //
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT    //
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE  //
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.   //
////////////////////////////////////////////////////////////////////////////

#ifndef I_SLS_SYSDEP
#define I_SLS_SYSDEP

#ifdef _WIN32
#  undef __x86__
#  define __x86__
#else
#  include <unistd.h>
#endif

/*---------------------------------------------------------------------------
  UNIX vs WIN32: sockets
  ---------------------------------------------------------------------------*/
#ifndef WIN32
#define SOCKET      int
#define BOOL        int
#define INVALID_SOCKET  -1
#define SOCKET_ERROR    -1
#define SOCKET_ERRNO    errno
#define SOCKET_IOCTL    ioctl
#define SOCKET_CLOSE    close
#else
#define SOCKET_ERRNO    WSAGetLastError()
#define EWOULDBLOCK WSAEWOULDBLOCK
#define SOCKET_IOCTL    ioctlsocket
#define SOCKET_CLOSE    closesocket

#define MAXHOSTNAMELEN  64
#endif

#ifndef WIN32
#define GETTIMEOFDAY(p) gettimeofday(p, NULL);
#else
#define GETTIMEOFDAY(p) gettimeofday(p);
#endif

/*---------------------------------------------------------------------------
  Sun
  ---------------------------------------------------------------------------*/

#ifdef  __sparc__
#undef  __LITTLE_ENDIAN__
#undef  __BIG_ENDIAN__
#undef  __BYTE_ORDER__
#define __LITTLE_ENDIAN__   0
#define __BIG_ENDIAN__      1
#define __BYTE_ORDER__
#endif /* __sparc__ */

#if defined(__i86pc__) || defined(__x86__) || defined(__x86_64__) || defined(__ia64__)
#undef  __LITTLE_ENDIAN__
#undef  __BIG_ENDIAN__
#undef  __BYTE_ORDER__
#define __LITTLE_ENDIAN__   1
#define __BIG_ENDIAN__      0
#define __BYTE_ORDER__
#endif /* defined(__i86pc__) || defined(__x86__) || defined(__x86_64__) || defined(__ia64__) */

#ifdef __sunos__
void  usleep(unsigned useconds);          /* sleep for some number of micro-seconds */
char *getwd(char *pathname);              /* get the current working direcotry */
int   gethostname(char *name, int namelen);   /* get the current host name */
int   shutdown(int s, int how);           /* for closing a socket */
int   listen(int s, int backlog);
#ifdef __sys_types_h
#ifdef _sys_socket_h
int   socket(int domain, int type, int protocol); /* open a socket */
int   bind(int s, struct sockaddr *name, int namelen);
int   accept(int s, struct sockaddr *name, int *addrlen);
int   connect(int s, struct sockaddr *name, int namelen);
int   getsockname(int s, struct sockaddr *name, int *namelen);
#endif /* #ifdef _sys_socket_h */
#ifdef _sys_time_h
int   select(int width, fd_set *readfds, fd_set *writefds,
         fd_set *exceptfds, struct timeval *timeout);
int   gettimeofday(struct timeval *tp, struct timezone *tzp);
#endif /* #ifdef _sys_time_h */
#ifdef _sys_timeb_h
int ftime(struct timeb *tp);
#endif /* #ifdef _sys_timeb_h */
#endif /* #ifdef __sys_types_h */
void  bcopy(char *b1, char *b2, int length);      /* copy in memory, taking caution w/ overlaps */
void  bzero(char *b, int length);         /* zero blocks of memory */
int   srandom(int);               /* seed random # generator */
long  random(void);               /* generate a random number */
int   strncasecmp(char *s1, char *s2, int n);     /* compare strings, upto length n, ignoring case */
void  swab(char *from, char *to, int nbytes);     /* swap adjacent bytes */
void  nice(int level);                /* renice a process */
int   getpgrp(int pid);               /* get process group id of a process*/
int   killpg(int pgrp, int sig);          /* send signal to an entire process group */
int   getpid(void);               /* return process id */
double drand48(void);                 /* get a random number */
void  srand48(long seedval);              /* seed random number generator */

int setsockopt(int s, int level, int optname, char *optval, int optlen);
int ioctl(int fd, int request, ...);
int recv(int s, char *buf, int len, int flags);


/* GCC 2.8 defines strcasecmp */
int   strcasecmp(char *s1, char *s2);         /* compare strings, ignoring case */

extern char *sys_errlist[];

/* implemented in compat/sunos */
/* SAM 8/24/99: strerror is defined in sysdep.c, but wasn't defined here. */
char *strtok_r(char *s1, const char *s2, char **lasts);
char *strerror(int errnum);

#endif /* __sunos__ */

#ifdef __solaris__

#include <stdlib.h>               /* for rand() in macro below */
#include <string.h>               /* for memset(), memmove() in macro below */

#undef _AFUNCS_H_
#define _AFUNCS_H_                /* keep <AF/Afuncs.h> from being included
                             (conflicts with our bzero/bcopy below */

/* implemented in compat/common */

double cospi(double x);
int irint(double x);
double anint(double x);
int nint(double x);
double exp10(double x);
double exp2(double x);
int isinf(double x);
int isnormal(double x);

/* defined in <ieeefp.h>, but won't include that yet */
int   finite(double);

#include <strings.h>
/* strings.h has these in Solaris 2.5.1
#define index(s, c)     (strchr(s, c))
#define rindex(s, c)        (strrchr(s, c))
#define bzero(sp, len)      ((void) memset(sp, 0, (size_t)(len)))
#define bcopy(s1, s2, len)  ((void) memmove(s2, s1, (size_t)(len)))
*/

/* implemented in compat/solaris */
int usleep(unsigned useconds);
int gethostname(char *name, int namelen);
char  *getwd(char *pathname);

extern char *sys_errlist[];

/* missing declarations */
#ifdef _SYS_TIMEB_H
int ftime(struct timeb *tp);
#endif /* #ifdef _SYS_TIMEB_H */

/* Defined in Solaris now, but no headers */
long random();
void srandom( unsigned seed);

#endif /* __solaris__ */

/*------------------------------------------------------------------------------------------------------------
  HP-UX
  ------------------------------------------------------------------------------------------------------------*/

#ifdef __hpux__

#include <stdlib.h>               /* for rand() in macro below */
#include <strings.h>                  /* for bcopy() and bzero() */

#undef __LITTLE_ENDIAN__
#undef __BIG_ENDIAN__
#define __LITTLE_ENDIAN__   0
#define __BIG_ENDIAN__      1

/* implemented in compat/common */

double cospi(double x);
int irint(double x);
double anint(double x); 
int nint(double x);
double exp10(double x);
double exp2(double x);
int isnormal(double x);
long random();
void srandom(int seed);

/* implemented in sysdep.c */
int usleep(unsigned useconds);
double rint(double x);

#endif /* __hpux__ */

/*---------------------------------------------------------------------------
  DEC Alpha OSF
  ---------------------------------------------------------------------------*/

#ifdef __alpha__

#undef __LITTLE_ENDIAN__
#undef __BIG_ENDIAN__
#define __LITTLE_ENDIAN__   1
#define __BIG_ENDIAN__      0

/* implemented in compat/common */

double cospi(double x);
int irint(double x);
double anint(double x); 
double exp10(double x);
double exp2(double x);
int isinf(double x);
int isnormal(double x);

/* missing declarations */
int close(int);
int ioctl(int fd, int request, ...);
void usleep(unsigned useconds);

#endif /* __alpha__ */

/*---------------------------------------------------------------------------
  NeXT
  ---------------------------------------------------------------------------*/

#ifdef __nextstep__

#include <libc.h>

/* __LITTLE_ENDIAN__ and __BIG_ENDIAN__ set automatically */

/* implemented in compat/common */

double cospi(double x);
int irint(double x);
double anint(double x);
int nint(double x);
double exp10(double x);
int isinf(double x);
int isnormal(double x);
int isnand(double dsrc);

#undef S_ISDIR
#undef S_ISLNK
#undef S_IXUSR
#define S_ISDIR(m)  (m && S_IFDIR)
#define S_ISLNK(m)  (m && S_IFLNK)
#define S_IXUSR     1

/* implemented in compat/nextstep */

char *strdup(char *str);
double  drand48(void);
void srand48(long seedval);

#endif /* __nextstep__ */
 
/*---------------------------------------------------------------------------
  Linux
  ---------------------------------------------------------------------------*/
 
#ifdef __linux__

#undef __LITTLE_ENDIAN__
#undef __BIG_ENDIAN__

#if __BYTE_ORDER == __LITTLE_ENDIAN
#define __LITTLE_ENDIAN__ 1
#define __BIG_ENDIAN__    0
#else
#define __LITTLE_ENDIAN__ 0
#define __BIG_ENDIAN__    1
#endif

#include <string.h>

/* implemented in compat/common */

double cospi(double x);
int irint(double x);
double anint(double x);
int nint(double x);
#ifdef __cplusplus
/* G++ defines _GNU_SOURCE which defines __USE_GNU which may define
 * a version of exp10 with the throw()
 */
double exp10(double x) throw ();
#else
double exp10(double x);
#endif

#endif /* __linux__ */

/*---------------------------------------------------------------------------
  SGI Irix
  ---------------------------------------------------------------------------*/
 
#ifdef __irix__

#include <strings.h>

#undef __LITTLE_ENDIAN__
#undef __BIG_ENDIAN__
#define __LITTLE_ENDIAN__ 0
#define __BIG_ENDIAN__    1

/* implemented in compat/common */

double cospi(double x);
int irint(double x);
double anint(double x);
int nint(double x);
double exp10(double x);
int isinf(double x);
int isnormal(double x);

#include <sys/timeb.h>
/* implemented in compat/irix */

int ftime(struct timeb *tbp);

#endif /* __irix__ */

/*---------------------------------------------------------------------------
  Windows/Windows NT
  ---------------------------------------------------------------------------*/
#ifdef WIN32
/*
 * Define some compatibility macros for WIN32/NT
 */

#undef __LITTLE_ENDIAN__
#undef __BIG_ENDIAN__
#define __LITTLE_ENDIAN__ 1
#define __BIG_ENDIAN__ 0

/* These are for printing out the file location. */

#undef str
#undef estr
//#undef __FUNCTION__
#define str(x) # x
#define estr(x) str(x)
//#define __FUNCTION__ __FILE__":"estr(__LINE__)

/* implemented in compat/win32 */

#include <string.h>
#define strcasecmp _stricmp
_CRTIMP char * __cdecl strdup(const char *);
void bcopy(const void *s1, void *s2, size_t n);
void bzero(void *s, size_t n);
char *strtok_r(char *s1, const char *s2, char **lasts);

#include <float.h>

int irint(double x);
int isinf(double x);
int isnan(double dsrc);

#include <sys/timeb.h>
#include <sys/types.h>

/** SPC Quote from MSDN Docs
 * You have to be careful when using Sleep and code that directly 
 * or indirectly creates windows. If a thread creates any windows, 
 * it must process messages. Message broadcasts are sent to all windows 
 * in the system. If you have a thread that uses Sleep with infinite 
 * delay, the system will deadlock. Two examples of code that indirectly
 * creates windows are DDE and COM CoInitialize. Therefore, if you have
 * a thread that creates windows, use MsgWaitForMultipleObjects or 
 * MsgWaitForMultipleObjectsEx, rather than Sleep.
 */
unsigned sleep(unsigned seconds);
void usleep(unsigned long useconds);

#include <io.h>
#include <stdio.h>
/* file io functions */
FILE *popen(const char *command, const char *type);
int pclose (FILE *stream);

_CRTIMP int __cdecl read(int, void *, unsigned int);
_CRTIMP int __cdecl write(int, const void *, unsigned int);

#include <process.h>
_CRTIMP int __cdecl getpid(void);

void gettimeofday(struct timeval *tp);

/* implemented in compat/common */
long random();
void srandom(int seed);

typedef unsigned char u_char;

#ifndef PURIFY
#undef purify_is_running
#undef purify_set_pool_id
#undef purify_map_pool
#define purify_is_running() FALSE
#define purify_set_pool_id(x, y)    /* stubbed out for now */
#define purify_map_pool(x, y)   /* stubbed out for now */
#endif

/* define a bunch of constants derived from pi, they are all defined in
 * <math.h> under UNIX, but not under NT. */
#undef M_PI
#undef M_PI_2
#undef M_PI_4
#undef M_PIl
#undef M_PI_2l
#undef M_PI_4l
#define M_PI            3.14159265358979323846  /* pi */
#define M_PI_2          1.57079632679489661923  /* pi/2 */
#define M_PI_4          0.78539816339744830962  /* pi/4 */
#define M_PIl           3.14159265358979323846L /* pi */
#define M_PI_2l         1.57079632679489661923L /* pi/2 */
#define M_PI_4l         0.78539816339744830962L /* pi/4 */

#endif  /* WIN32 */

#endif /* I_SLS_SYSDEP */

/* 
  for Emacs...
  Local Variables:
  mode: C
  comment-column: 50
  fill-column: 110
  c-indent-level: 2
  c-continued-statement-offset: 2
  c-brace-offset: -2
  c-argdecl-indent: 2
  c-label-offset: -2
  End:
*/

