// -*- C++ -*-
// Copyright (c) 2006
// Massachusetts Institute of Technology
// All Rights Reserved

#ifndef __BitStreamReader32_h__
#define __BitStreamReader32_h__

#include "IntTypes.h"

class BitStreamReader32 {

    const u32* _ptr;
    s32        _bitPos;
    const u32* _begin;

public:

    BitStreamReader32()
        : _ptr(0), _bitPos(0), _begin(0) {
    }

    BitStreamReader32(const u32* begin) {
        initialize(begin);
    }

    void initialize(const u32* begin) {
        _ptr = begin;
        _bitPos = 32;
        _begin = begin;
    }

    u32 read(u32 resultBits) {
        u32 result = (*_ptr << (32 - _bitPos)) >> (32 - resultBits);
        _bitPos -= resultBits;
        if (_bitPos == 0) {
            _bitPos += 32;
            _ptr++;
        } else if (_bitPos < 0) {
            _bitPos += 32;
            _ptr++;
            result |= *_ptr >> _bitPos;
        }
        return result;
    }

    s32 readSigned(u32 resultBits) {
        u32 x = read(resultBits);
        return -s32(x & (1 << (resultBits - 1))) | x; // sign extend
    }

    u32 position() const {
        return 32*(_ptr - _begin + 1) - _bitPos;
    }

    void seek(u32 position) {
        _ptr = _begin + position/32;
        _bitPos = 32 - position%32;
    }
};

#endif // __BitStreamReader32_h__

// Local Variables:
// mode: C++
// comment-column: 48
// fill-column: 79
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
