// -*- C++ -*-

#ifndef __BitStreamWriter32_h__
#define __BitStreamWriter32_h__

#include <string>
#include <vector>
#include "IntTypes.h"

class BitStreamWriter32 {

protected:
    u32              _buf;
    u32              _p;
    std::vector<u32> _words;
    bool             _do_swap;

public:
    BitStreamWriter32(bool swap=false)
        : _buf(0), _p(32), _do_swap(swap) {
    }

    void write(u32 value, u32 valueBits) {
        do {
            value <<= 32 - valueBits;
            value >>= 32 - valueBits;
            s32 shift = _p - valueBits;
            if (shift >= 0) {
                _buf |= value << shift;
                _p = shift;
                valueBits = 0;
            } else {
                _buf |= value >> -shift;
                valueBits = -shift;
                _p = 0;
            }
            if (_p == 0) {
                _words.push_back(_swap(_buf));
                _buf = 0;
                _p = 32;
            }
        } while (valueBits);
    }

    void write(s32 value, u32 valueBits) {
        write(u32(value), valueBits);
    }

    void pad(u32 n) {
        int p = position();
        int m = p % n;
        if (m != 0) {
            write(0, n - m);
        }
    }

    void flush() {
        if (_p != 32) {
            _words.push_back(_swap(_buf));
        }
    }

    u32 position() const {
        return 32*_words.size() + 32 - _p;
    }

    u32 length() const {
        return 4*_words.size();
    }

    const u8* data() const {
        return (u8*) &_words.front();
    }

protected:

    u32 _swap(u32 x) {
        if (_do_swap) {
            x = ((x >> 8) & 0x00ff00ff) | ((x & 0x00ff00ff) << 8);
            x = ( x >> 16             ) | ( x               << 16);
        }
        return x;
    }
};

#endif // __BitStreamWriter32_h__
