/* -*- C++ -*-
 *
 * Copyright (c) 2007
 * Spoken Language Systems Group
 * MIT Computer Science and Artificial Intelligence Laboratory
 * Massachusetts Institute of Technology
 *
 * All Rights Reserved
 */

#include "fst/FST.h"

using namespace std;

typedef pair<int,int> Pair;
typedef fst::map<Pair,int> PairIntMap;
typedef fst::vector<Pair>  IntPairMap;
typedef fst::map<int,int> IntIntMap;
typedef fst::vector<int> IntVector;
typedef fst::set<int> IntSet;

static int
delayOutputsInsert(PairIntMap& pairIntMap, IntPairMap& intPairMap, FSTGeneric& out,
                   int state, int output)
{
    Pair stateOutput(state, output);
    pair<PairIntMap::iterator,bool> status =
        pairIntMap.insert(PairIntMap::value_type(stateOutput, pairIntMap.size()));
    if (status.second) {
        intPairMap.push_back(stateOutput);
        out->add_state();
    }
    return (*status.first).second;
}

FSTGeneric
FSTDelayOutputs(const FSTGeneric& in)
{
    PairIntMap pairIntMap;
    IntPairMap intPairMap;

    FSTGeneric out = FSTBasic();
    out->set_initial
        (delayOutputsInsert(pairIntMap, intPairMap, out, in->initial(), 0));
    out->set_final
        (delayOutputsInsert(pairIntMap, intPairMap, out, -1, 0));
    for (unsigned q = 0; q < intPairMap.size(); ++q) {
        Pair po = intPairMap[q];
        int p = po.first;
        int o = po.second;
        if (p == -1) {
            continue;
        }
        if (in->is_final(p)) {
            if (o) {
                out->add_arc(q, 1, 1, o, in->final_weight(p));
            } else {
                out->set_final(q, in->final_weight(p));
            }
        }
        for (FSTArcIter a = in->arcs_out(p); a.valid(); ++a) {
            assert(o == 0 || a->output == 0);
            if (a->input == 1) {
                int output = o | a->output;
                int r = delayOutputsInsert(pairIntMap, intPairMap, out, a->next, 0);
                out->add_arc(q, r, a->input, output, a->weight);
            } else {
                int pending = o | a->output;
                int r = delayOutputsInsert(pairIntMap, intPairMap, out, a->next, pending);
                out->add_arc(q, r, a->input, 0, a->weight);
            }
        }
    }

    return out;
}

// for Emacs...
// Local Variables:
// mode: C++
// comment-column: 48
// fill-column: 80
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
