////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2005, Massachusetts Institute of Technology              //
// All rights reserved.                                                   //
//                                                                        //
// Redistribution and use in source and binary forms, with or without     //
// modification, are permitted provided that the following conditions are //
// met:                                                                   //
//                                                                        //
//     * Redistributions of source code must retain the above copyright   //
//       notice, this list of conditions and the following disclaimer.    //
//                                                                        //
//     * Redistributions in binary form must reproduce the above          //
//       copyright notice, this list of conditions and the following      //
//       disclaimer in the documentation and/or other materials provided  //
//       with the distribution.                                           //
//                                                                        //
//     * Neither the name of the Massachusetts Institute of Technology    //
//       nor the names of its contributors may be used to endorse or      //
//       promote products derived from this software without specific     //
//       prior written permission.                                        //
//                                                                        //
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS    //
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT      //
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR  //
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT   //
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,  //
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT       //
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,  //
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY  //
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT    //
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE  //
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.   //
////////////////////////////////////////////////////////////////////////////

// Factor FST into a bimachine Tl/Tr, such that T o X = Tl o reverse(Tr o reverse(X)).
//
// This is not a minimal factoring, but it is fairly straightforward.  Roche has
// published a smaller factorization in a MERL TR.

#include "fst/FST_i.h"
#include <sstream>

using namespace std;

void
FSTFactorRL(const FSTGeneric& in, FSTGeneric& R, FSTGeneric& L)
{
    FSTGeneric y = FSTRemoveEpsilonsA(FSTProject(FSTReverse(FSTClearWeights(in)), FSTOutput, FSTOutput));
    FSTGeneric x = FSTDeterminizeA(FSTBasic(y));
    FSTBasic(x);

    R = FSTBasic(x->semiring());
    R->add_states(x->n_states());
    R->set_initial(x->initial());

    typedef fst::set<int> IntSet;
    typedef fst::map<IntSet,int> IntSetMap;
    IntSetMap set_map;
    int n = 0;

    for (int p = 0; p < x->n_states(); ++p) {
        FSTArcIter a = x->arcs_out(p);
        IntSet det_set, re_set, det_re_set;
        FSTDeterminizeStateSet(x, p, det_set);
        for (IntSet::const_iterator i = det_set.begin(); i != det_set.end(); ++i) {
            IntSet re_set;
            FSTRemoveEpsilonsStateSet(y, *i, re_set);
            for (IntSet::const_iterator j = re_set.begin(); j != re_set.end(); ++j) {
                det_re_set.insert(*j);
            }
        }

        ostringstream os;
        pair<IntSetMap::iterator,bool> result =
            set_map.insert(IntSetMap::value_type(det_re_set, n));
        if (result.second)
            ++n;
        os << '.' << (*result.first).second;
        string suffix = os.str();

        R->set_final(p, x->final_weight(p));

        for (; a.valid(); ++a) {
            string input = FSTstring_map[a->output] + suffix;
            R->add_arc(p, a->next, FSTstring_map[input], a->output);
        }

    }

    R = FSTOptimizeA(R);

    L = FSTOptimizeA(FSTReverse(FSTCompose(FSTOptimizeA(FSTReverse(in)), FSTProject(R, FSTOutput, FSTInput))));
}

// for Emacs...
// Local Variables:
// mode: C++
// comment-column: 48
// fill-column: 80
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
