/* -*- C++ -*-
 *
 * Copyright (c) 2007
 * Spoken Language Systems Group
 * MIT Computer Science and Artificial Intelligence Laboratory
 * Massachusetts Institute of Technology
 *
 * All Rights Reserved
 */

#error UNMODIFIED FROM SLS VERSION, SO NEEDS FIXING/MERGING WITH FST TREE.

#define VERBOSE 0

#include <iostream>
#include <sstream>
#include <fstream>
#include <cstdlib>
#include <cassert>
#include "sls/FST.h"
#include "sls/map"
#include "sls/sq.h"
#include "sls/BitStreamWriter32.h"

// echo -n PDFA1 | md5sum | cut -c1-8
#define PDFA1         0x575a24eb
#define PDFA1_SWAPPED 0xeb245a57

// echo -n PFST1 | md5sum | cut -c1-8
#define PFST1         0xc420ef26
#define PFST1_SWAPPED 0x26ef20c4

// echo -n QFST1 | md5sum | cut -c1-8
#define QFST1         0xff8e81bf
#define QFST1_SWAPPED 0xbf818eff

#define F2Q(X,Q) u32(rintf(X * (1 << Q)))

using namespace std;

typedef std::pair<int,int> Pair;
typedef sls::set<Pair> PairSet;
typedef sls::map<Pair,int> PairIntMap;
typedef sls::vector<Pair>  IntPairMap;
typedef sls::map<int,int> IntIntMap;
typedef sls::vector<int> IntVector;
typedef sls::set<int> IntSet;

// Compute number of bits needed to encode unsigned integer n.
//
// Derived from:
//   http://graphics.stanford.edu/~seander/bithacks.html#VariableSignExtend
//   http://graphics.stanford.edu/~seander/bithacks.html#CountBitsSetKernighan
//
static unsigned
bitsNeeded(unsigned n)
{
    unsigned c;
    n |= n >> 1;
    n |= n >> 2;
    n |= n >> 4;
    n |= n >> 8;
    n |= n >> 16;
    for (c = 0; n; c++)
        n &= n - 1;
    return c;
}

static unsigned
nextHighestPowerOf2(unsigned v)
{
    v--;
    v |= v >> 1;
    v |= v >> 2;
    v |= v >> 4;
    v |= v >> 8;
    v |= v >> 16;
    v++;
    return v;
}

static u32
swap32(u32 x)
{
    x = ((x >> 8) & 0x00ff00ff) | ((x & 0x00ff00ff) << 8);
    x = ( x >> 16             ) | ( x               << 16);
    return x;
}

static u16
swap16(u16 x)
{
    x = (x >> 8) | (x << 8);
    return x;
}

static void
dump_u32(FILE* fp, int x, bool swap)
{
    if (swap) x = swap32(x);
    fwrite(&x, sizeof(x), 1, fp);
}

static void
dump_u16(FILE* fp, int x, bool swap)
{
    assert((x >> 16) == 0);
    u16 y(x);
    if (swap) y = swap16(y);
    fwrite(&y, sizeof(y), 1, fp);
}

static void
dump_s16(FILE* fp, int x, bool swap)
{
    assert((x >> 16) == 0 || (x >> 16) == -1);
    s16 y(x);
    if (swap) y = swap16(y);
    fwrite(&y, sizeof(y), 1, fp);
}

static void
dump_u8(FILE* fp, int x)
{
    assert((x & 0xffffff00) == 0);
    u8 y(x);
    fwrite(&y, sizeof(y), 1, fp);
}

static void 
dump_pad(FILE* fp, int n)
{
    while (ftell(fp) & (n - 1)) {
        dump_u8(fp, 0);
    }
}

// Main entry point.
//
void
FSTPackToFile(const char* filename, const FSTGeneric& in, bool swap)
{
    // This is hard-coded in PFST loading/walking code inside PocketSUMMIT.  Do
    // not change.
    //
    const u32 weightBits = 5;

    // Topologically sort with respect to null (epsilon and #) transitions only.
    //
    sls::set<int> sortInputs;
    sortInputs.insert(0);
    sortInputs.insert(1);
    FSTGeneric fst = FSTTopologicalSort(in, sortInputs);

    // Train Lloyd-Max (non-linear) weight quantizer.
    //
    void* trainer = sq_trainer_lloyd_max_new(weightBits);
    bool anyNonZero = false;
    for (int p = 0; p < fst->n_states(); ++p) {
        for (FSTArcIter a = fst->arcs_out(p); a.valid(); ++a) {
            if (a->weight != 0) {
                sq_trainer_incorporate(trainer, a->weight, 1);
                anyNonZero = true;
            }
        }
    }
    float* weightD = 0;
    float* weightR = 0;
    if (anyNonZero) {
        sq_trainer_train(trainer, &weightD, &weightR);
    }

    // Compute maximum index of internal "i(...)" label.
    //
    int internalMax = 0;
    int inputMax = 0;
    IntIntMap inputMap;
    const IntSet& inputs = fst->input_alphabet();
    for (IntSet::const_iterator i = inputs.begin(); i != inputs.end(); ++i) {
        const char* label = FSTstring_map[*i];
        if ((label[0] == 't' || label[0] == 'i') && label[1] == '.') {
            int index = atoi(label + 2) + 2;
            inputMap[*i] = index;
            //cerr << "inputMap[" << label << "] = " << index << '\n';
            if (label[0] == 'i' && index > internalMax) {
                internalMax = index;
            }
            if (index > inputMax) {
                inputMax = index;
            }
        }
    }
    inputMap[0] = 0;
    inputMap[1] = 1;
    cerr << "internalMax = " << internalMax << '\n';

    u32 nStates = fst->n_states();
    //u32 nArcs = fst->n_arcs();
    u32 nInputs = inputMax + 1;

    IntSet outputs = fst->output_alphabet();
    outputs.insert(0);

    typedef std::map<string,int> StringIntMap;
    StringIntMap outputStringMap;
    for (IntSet::const_iterator i = outputs.begin(); i != outputs.end(); ++i) {
        outputStringMap[FSTstring_map[*i]] = 0;
    }
    int outputIndex = 0;
    for (StringIntMap::iterator i = outputStringMap.begin();
         i != outputStringMap.end();
         ++i) {
        (*i).second = outputIndex++;
    }
    IntIntMap outputIntMap;
    for (IntSet::const_iterator i = outputs.begin(); i != outputs.end(); ++i) {
        outputIntMap[*i] = outputStringMap[FSTstring_map[*i]];
    }

    u32 inputBits = bitsNeeded(nInputs);
    u32 stateBits = bitsNeeded(nStates);
    u32 outputBits = bitsNeeded(outputs.size());

    cout << "bit widths:"
         << " input=" << inputBits
         << " output=" << outputBits
         << " next=" << stateBits
         << '\n';

    // Pack all arcs, keeping track of bit offsets for the first arc leaving
    // each states.
    //
    u32 dInput = 0;
    u32 dNext = 0;
    u32 dOutput = 0;
    u32 dWeight = 0;
    BitStreamWriter32 arcWriter(swap);
    sls::vector<u32> positionForState;
    for (u32 p = 0; p < nStates; ++p) {
        positionForState.push_back(arcWriter.position());

        if (fst->is_final(p))
            cout << "final state " << p << " at position " << arcWriter.position() << '\n';

        // state finality
        arcWriter.write(fst->is_final(p), 1);

        for (FSTArcIter a = fst->arcs_out_with_input(p, 0, FSTGreaterEqual); a.valid(); ++a) {

            // flag not finished with arcs
            arcWriter.write(1, 1);
            dInput++;

            // input
            assert(inputMap.count(a->input) != 0);
            int mappedInput = inputMap[a->input];
            assert((mappedInput >> inputBits) == 0);
            arcWriter.write(mappedInput, inputBits);
            dInput += inputBits;

            assert(a->output == 0 || a->input == 1);

#if 0
            //assert(!(mappedInput >= 2 && mappedInput <= internalMax) || a->next == p);
            if (!(!(mappedInput >= 2 && mappedInput <= internalMax) || a->next == p)) {
                cerr << "Illegal internal transition from " << p << " to " << a->next << ".\n";
                cerr << "See debug.fst.\n";
                fst->dump_ascii("debug.fst");
                assert(false);
            }
#endif

            assert(a->input > 1 || a->next > p);

            // DEBUG
            if (p == 0) {
                cerr << "T " << p << ' ' << a->next << ' ' << FSTstring_map[a->input] << ' ' << mappedInput << ' ' << (mappedInput < 2 || mappedInput > internalMax) << '\n';
            }

            if (mappedInput < 2 || mappedInput > internalMax) {

                // next state
                assert((a->next >> stateBits) == 0);
                arcWriter.write(a->next, stateBits);
                dNext += stateBits;

                // output (word)
                if (mappedInput == 1) {
                    int output = outputIntMap[a->output];
                    assert(!a->output || output);
                    assert(output < outputStringMap.size());
                    assert((output >> outputBits) == 0);
                    arcWriter.write(output, outputBits);
                    dOutput += outputBits;
                }

                // weight
                if (a->weight == 0) {
                    arcWriter.write(0, 1);
                    dWeight++;
                } else {
                    int weight = sq_quantize(a->weight, weightD, 1 << weightBits);
                    arcWriter.write(1, 1);
                    assert((weight >> 5) == 0);
                    arcWriter.write(weight, 5);
                    dWeight += 6;
                }

            } else { // internal

                if (a->next == p) {
                    arcWriter.write(0, 1); // no next state needed
                } else {
                    arcWriter.write(1, 1);
                    assert((a->next >> stateBits) == 0);
                    arcWriter.write(a->next, stateBits);
                }

            }
        }
        
        // flag finished with arcs
        arcWriter.write(0, 1);
        dInput++;
    }
    arcWriter.flush();
    positionForState.push_back(arcWriter.position());

    {
        u32 nArcs = fst->n_arcs();
        u32 nBits = arcWriter.position();
        cerr << "arcs: " << nBits << " bits, " << nArcs << " arcs, "
             << double(nBits)/double(nArcs) << " bits/arc\n";
    }

    // Determine shift to use with state table 1.
    //
    u32 tableBits = 16;
    u32 tableShift = 0;
    for (u32 shift = 16; shift > 0; --shift) {
        u32 cumArcBits = 0;
        bool ok = true;
        for (u32 p = 0; p < nStates; ++p) {
            if ((p & ((1 << shift) - 1)) == 0) {
                cumArcBits = 0;
            }
            u32 arcBits = positionForState[p + 1] - positionForState[p];
            u32 pos = cumArcBits + arcBits;
            if (pos >> tableBits) {
                ok = false;
                break;
            }
            cumArcBits += arcBits;
        }
        if (ok) {
            tableShift = shift;
            break;
        }
    }
    
    // Construct the state tables.
    //
    std::vector<u32> stateTable1;
    std::vector<u16> stateTable2;
    for (u32 p = 0, cumArcBits = 0; p < nStates; ++p) {
        if ((p & ((1 << tableShift) - 1)) == 0) {
            stateTable1.push_back(positionForState[p]);
            cumArcBits = 0;
        }
        assert((cumArcBits >> 16) == 0);
        stateTable2.push_back(cumArcBits);
        u32 arcBits = positionForState[p + 1] - positionForState[p];
        cumArcBits += arcBits;
    }
    cout << "state table shift: " << tableShift << '\n';

    // Construct the word table.
    //
    std::vector<u32> wordTable;
    u32 wordOffset = 0;
    for (StringIntMap::iterator i = outputStringMap.begin();
         i != outputStringMap.end();
         ++i) {
        wordTable.push_back(wordOffset);
        wordOffset += (*i).first.length() + 1;
    }
    cout << "word table size = " << wordTable.size() << '\n';

    ////////////////////////////////////////////////////////////////////////////
    // Output to file
    ////////////////////////////////////////////////////////////////////////////

    FILE* fp = fopen(filename, "wb");
    assert(fp);

    // version
    dump_u32(fp, PFST1, swap);

    // filesize (set later)
    u32 sizePos = ftell(fp);
    dump_u32(fp, 0, swap);

    // state table 1 pointer (set later)
    dump_u32(fp, 0, swap);

    // state table 2 pointer (set later)
    dump_u32(fp, 0, swap);

    // arc data pointer (set later)
    dump_u32(fp, 0, swap);

    // word table pointer (set later)
    dump_u32(fp, 0, swap);

    // word string pointer (set later)
    dump_u32(fp, 0, swap);

    // number of states
    dump_u32(fp, nStates, swap);
    
    // initial state
    dump_u32(fp, fst->initial(), swap);

    // max internal label
    dump_u16(fp, internalMax, swap);

    // state table shift
    dump_u8(fp, tableShift);

    // input bit width
    dump_u8(fp, inputBits);

    // next state bit width
    dump_u8(fp, stateBits);

    // output bit width
    dump_u8(fp, outputBits);

    // weight reconstruction values
    dump_pad(fp, 2);
    for (u32 i = 0; i < (1 << weightBits); ++i) {
        dump_s16(fp, weightR ? s16(F2Q(weightR[i], 6)) : 0, swap);
    }

    // state table 1
    dump_pad(fp, 32);
    u32 stateTable1Pos = ftell(fp);
    int debug_count = 4;
    cout << "debug:";
    int count = debug_count;
    for (std::vector<u32>::iterator i = stateTable1.begin(); i != stateTable1.end(); ++i) {
        if (count > 0) {
            cout << ' ' << *i;
            --count;
        }
        dump_u32(fp, *i, swap);
    }

    // state table 2
    dump_pad(fp, 32);
    u32 stateTable2Pos = ftell(fp);
    count = debug_count;
    for (std::vector<u16>::iterator i = stateTable2.begin(); i != stateTable2.end(); ++i) {
        if (count > 0) {
            cout << ' ' << *i;
            --count;
        }
        dump_u16(fp, *i, swap);
    }
    cout << '\n';

    // arc data
    dump_pad(fp, 32);
    u32 arcPos = ftell(fp);
    fwrite(arcWriter.data(), arcWriter.length(), 1, fp);

    // word table
    dump_pad(fp, 32);
    u32 wordTablePos = ftell(fp);
    for (std::vector<u32>::iterator i = wordTable.begin(); i != wordTable.end(); ++i) {
        dump_u32(fp, *i, swap);
    }

    // word strings
    dump_pad(fp, 32);
    u32 wordStringPos = ftell(fp);
    for (StringIntMap::iterator i = outputStringMap.begin();
         i != outputStringMap.end();
         ++i) {
        fwrite((*i).first.c_str(), (*i).first.length() + 1, 1, fp);
    }

    // fix up postponed pointers
    u32 size = ftell(fp);
    fseek(fp, sizePos, SEEK_SET);
    dump_u32(fp, size, swap);
    dump_u32(fp, stateTable1Pos, swap);
    dump_u32(fp, stateTable2Pos, swap);
    dump_u32(fp, arcPos, swap);
    dump_u32(fp, wordTablePos, swap);
    dump_u32(fp, wordStringPos, swap);
    cout << "stateTable1Pos = " << stateTable1Pos << '\n';
    cout << "stateTable2Pos = " << stateTable2Pos << '\n';

    // done
    fclose(fp);

    cout << "Wrote " << size << " bytes to file " << filename << ".\n";

    // dump vocabulary (alphabet) file
    string vocab_file(filename);
    vocab_file += ".vocab";
    fp = fopen(vocab_file.c_str(), "w");
    for (StringIntMap::iterator i = outputStringMap.begin();
         i != outputStringMap.end();
         ++i) {
        fputs((*i).first.c_str(), fp);
        fputc('\n', fp);
    }
}

// Pack deterministic, unweighted FSA to file.  This is currently used for
// as a filter for N-best in PocketSUMMIT (address domain).
//
// fixed-size state: 1-bit finality, A-bit arc index
// fixed-size arc:   S-bit symbol, N-bit next state
//
void
DFAPackToFile(const char* filename, const FSTGeneric& in, bool swap,
                           const char* vocabFilename)
{
    u32 N = in->n_states();

    // Construct symbolMap from vocabFilename.
    IntIntMap symbolMap;
    IntIntMap reverseSymbolMap;
    {
        ifstream is(vocabFilename);
        string line;
        int index = 0;
        while (getline(is, line)) {
            int fstIndex = FSTstring_map[line];
            symbolMap[fstIndex] = index;
            reverseSymbolMap[index] = fstIndex;
            ++index;
        }
    }

    // Determine bit widths.
    u32 arcIndexBits = bitsNeeded(in->n_arcs());
    u32 nextBits = bitsNeeded(N);
    u32 symbolBits = bitsNeeded(symbolMap.size());

#if VERBOSE
    cerr << "arcBits=" << arcIndexBits << " symbolBits=" << symbolBits << " nextBits=" << nextBits << '\n';
#endif

    // Pack arcs, sorting by symbol index, and checking for non-determinism.
    BitStreamWriter32 arcWriter(swap);
    IntVector arcIndices;
    arcIndices.reserve(N + 1);
    int arcIndex = 0;
    for (u32 p = 0; p < N; ++p) {
        arcIndices.push_back(arcIndex);
        PairSet pairs;
        for (FSTArcIter a = in->arcs_out(p); a.valid(); ++a) {
            assert(a->input > 1);
            int symbol = symbolMap[a->input];
            if (symbol != 0) {
                ++arcIndex;
                pairs.insert(Pair(symbol, a->next));
            }
        }
        int lastSymbol = 0;

#if VERBOSE
        cerr << "# " << arcIndices[p] << ' ' << arcIndex << ' ' << arcWriter.position() << '\n';
#endif

        for (PairSet::const_iterator i = pairs.begin(); i != pairs.end(); ++i) {
            int symbol = (*i).first;
            int next = (*i).second;

            assert(symbol > lastSymbol);
            lastSymbol = symbol;
#if VERBOSE
            cerr << "T " << p << ' ' << next << ' ' << symbol << ' ' << FSTstring_map[reverseSymbolMap[symbol]] << '\n';
#endif
            arcWriter.write(symbol, symbolBits);
            arcWriter.write(next, nextBits);
        }
    }
    arcIndices.push_back(arcIndex);

    // Pack states.
    BitStreamWriter32 stateWriter(swap);
    for (u32 p = 0; p < N; ++p) {
        stateWriter.write(in->is_final(p), 1);
        stateWriter.write(arcIndices[p], arcIndexBits);
    }
    stateWriter.write(0, 1);
    stateWriter.write(arcIndices[N], arcIndexBits);

    // Output to file.

    FILE* fp = fopen(filename, "wb");
    assert(fp);

    // version
    dump_u32(fp, PDFA1, swap);

    // filesize (set later)
    u32 sizePos = ftell(fp);
    dump_u32(fp, 0, swap);

    // state table (set later)
    dump_u32(fp, 0, swap);

    // arc table (set later)
    dump_u32(fp, 0, swap);

    // number of states
    dump_u32(fp, N, swap);

    // arc index bits
    dump_u8(fp, arcIndexBits);

    // symbol bits
    dump_u8(fp, symbolBits);

    // next state bits
    dump_u8(fp, nextBits);

    // state table
    dump_pad(fp, 4);
    u32 statePos = ftell(fp);
    fwrite(stateWriter.data(), stateWriter.length(), 1, fp);

    // arc table
    dump_pad(fp, 4);
    u32 arcPos = ftell(fp);
    fwrite(arcWriter.data(), arcWriter.length(), 1, fp);

    // fix up postponed pointers
    u32 size = ftell(fp);
    fseek(fp, sizePos, SEEK_SET);
    dump_u32(fp, size, swap);
    dump_u32(fp, statePos, swap);
    dump_u32(fp, arcPos, swap);

    // done
    fclose(fp);

    cerr << "Wrote " << size << " bytes to file " << filename << ".\n";
}


// for Emacs...
// Local Variables:
// mode: C++
// comment-column: 48
// fill-column: 80
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
