#include <iostream>
#include "fst/FST.h"
#include "sq.h"

using namespace std;

static FSTGeneric
_FSTQuantizeWeights(int bits, const FSTGeneric& in)
{
    int n = 1 << bits;
    FSTSemiring semiring = in->semiring();

    void* trainer = sq_trainer_lloyd_max_new(bits);
    for (int p = 0; p < in->n_states(); ++p) {
        float fw = in->final_weight(p);
        if (fw != semiring->zero) {
            sq_trainer_incorporate(trainer, fw, 1);
        }
        for (FSTArcIter a = in->arcs_out(p); a.valid(); ++a) {
            float aw = a->weight;
            if (aw != semiring->zero) {
                sq_trainer_incorporate(trainer, aw, 1);
            }
        }
    }

    cerr << "training quantizer" << flush;
    float *d, *r;
    sq_trainer_train(trainer, &d, &r);
    cerr << "done\n";


    cerr << "quantizing" << flush;
    FSTGeneric out = FSTBasic(semiring);
    out->add_states(in->n_states());
    out->set_initial(in->initial());
    for (int p = 0; p < in->n_states(); ++p) {
        if ((p % 1000) == 0) {
            cerr << '.' << flush;
        }
        float fw = in->final_weight(p);
        if (fw != semiring->zero) {
            fw = r[sq_quantize(fw, d, n)];
        }
        out->set_final(p, fw);
        for (FSTArcIter a = in->arcs_out(p); a.valid(); ++a) {
            float aw = a->weight;
            if (aw != semiring->zero) {
                aw = r[sq_quantize(aw, d, n)];
            }
            out->add_arc(p, a->next, a->input, a->output, aw);
        }
    }
    cerr << "done.\n";

    sq_trainer_delete(trainer);

    return out;
}

FSTGeneric
FSTQuantizeWeights(int bits, FSTGeneric& in)
{
    FSTNgramInt* ngram = dynamic_cast<FSTNgramInt*>(in.pointer());
    if (ngram) {
        ngram->quantize(bits);
        return in;
    } else {
        return _FSTQuantizeWeights(bits, in);
    }
}
