////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2005, Massachusetts Institute of Technology              //
// All rights reserved.                                                   //
//                                                                        //
// Redistribution and use in source and binary forms, with or without     //
// modification, are permitted provided that the following conditions are //
// met:                                                                   //
//                                                                        //
//     * Redistributions of source code must retain the above copyright   //
//       notice, this list of conditions and the following disclaimer.    //
//                                                                        //
//     * Redistributions in binary form must reproduce the above          //
//       copyright notice, this list of conditions and the following      //
//       disclaimer in the documentation and/or other materials provided  //
//       with the distribution.                                           //
//                                                                        //
//     * Neither the name of the Massachusetts Institute of Technology    //
//       nor the names of its contributors may be used to endorse or      //
//       promote products derived from this software without specific     //
//       prior written permission.                                        //
//                                                                        //
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS    //
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT      //
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR  //
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT   //
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,  //
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT       //
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,  //
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY  //
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT    //
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE  //
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.   //
////////////////////////////////////////////////////////////////////////////

#include "fst/FST_i.h"
#include "fst/FSTStringMap.h"
#include "fst/format.h"
#include <fstream>
#include <cstdlib>

using std::string;
using std::ifstream;
using std::cerr;

FSTStringMap::FSTStringMap() :
#ifdef FSTSTRINGMAP_USE_HASH
    fmap(50000),
#else
    fmap(),
#endif
    rmap(), n(0)
{
    // make sure "" (epsilon) is 0
    operator[]("");
    // make sure "#" (word boundary) is 1
    operator[]("#");

    // preload labels from a file
    char* preload_file = getenv("FST_LABELS_FILE");
    if (preload_file) {
        string label;
        ifstream is(preload_file);
        if (is.good()) {
            int n = 0;
            while (!is.eof() && is.good()) {
                is >> label;
                operator[](label);
                n++;
            }
            format(cerr, "preloaded %d labels from %s\n", n, preload_file);
        }
    }
}

FSTStringMap::~FSTStringMap()
{
}

int
FSTStringMap::operator[](const char* s)
{
    return operator[](string(s));
}

int
FSTStringMap::operator[](const string& s)
{
    int result;
#ifdef FSTSTRINGMAP_USE_HASH
    _Map::iterator i = fmap.find(s);
    if (i != fmap.end()) {
        result = (*i).second;
    } else {
        result = n++;
        rmap.push_back(s);
        fmap.insert(_Map::value_type(s, result));
    }
#else
    _Map::iterator i = fmap.lower_bound(s);
    if (i != fmap.end() && (*i).first == s) {
        // found
        result = (*i).second;
    } else {
        // not found -- new entry
        result = n++;
        rmap.push_back(s);
        fmap.insert(i, _Map::value_type(s, result));
    }
#endif
    return result;
}

const char *
FSTStringMap::operator[](int i) const
{
    return rmap[i].c_str();
}

int
FSTStringMap::size() const
{
    return n;
}

// for Emacs...
// Local Variables:
// mode: C++
// comment-column: 50
// fill-column: 110
// c-basic-offset: 4
// End:
