#ifndef __Heap_h__
#define __Heap_h__

template<class _Type, class _Less>
class Heap {
public:
    Heap()
        : _rep(0), _size(0), _capacity(0) {
    }
    Heap(const _Less& less)
        : _rep(0), _size(0), _capacity(0), _less(less) {
    }
    Heap(const _Less& less, unsigned capacity)
        : _rep((new _Type*[capacity]) - 1), _size(0), _capacity(capacity), _less(less) {
    }
    unsigned size() const {
        return _size;
    }
    unsigned capacity() const {
        return _capacity;
    }
    _Type* top() {
        return _size > 0 ? _rep[1] : 0;
    }
    void pop() {
        if (_size > 1) {
            _rep[1]->set_heap_index(0);
            _rep[1] = _rep[_size];
            _rep[1]->set_heap_index(1);
            _rep[_size] = 0;
            _size--;
            _heapify_down(1);
        } else if (_size == 1) {
            _rep[1]->set_heap_index(0);
            _rep[1] = 0;
            _size = 0;
        }
    }
    void pop_and_insert(_Type* item) {
        item->set_heap_index(1);
        _rep[1] = item;
        _heapify_down(1);
    }
    void insert(_Type* item) {
        if (_size == _capacity) {
            if (_capacity == 0)
                _capacity = 1;
            else
                _capacity *= 2;
            _Type** new_rep = (new _Type*[_capacity]) - 1;
            for (unsigned i = 1; i <= _size; i++)
                new_rep[i] = _rep[i];
            if (_size > 0)
                delete[] (_rep + 1);
            _rep = new_rep;
        }
        _size++;
        item->set_heap_index(_size);
        _rep[_size] = item;
        _heapify_up(_size);
    }
    void adjust(const _Type* item) {
        unsigned i = item->get_heap_index();
        if (!_heapify_up(i))
            _heapify_down(i);
    }
    void increase(const _Type* item) {
        _heapify_down(item->get_heap_index());
    }
    void decrease(const _Type* item) {
        _heapify_up(item->get_heap_index());
    }
    _Type* get(unsigned i) {
        return _rep[i];
    }

protected:
    _Type** _rep;
    unsigned _size;
    unsigned _capacity;
    _Less _less;

    unsigned _parent(unsigned i) {
        return i/2;
    }

    unsigned _left(unsigned i) {
        return i*2;
    }

    unsigned _right(unsigned i) {
        return i*2 + 1;
    }
    void _exchange(unsigned i, unsigned j) {
        _Type* tmp = _rep[i];
        _rep[i] = _rep[j];
        _rep[i]->set_heap_index(i);
        _rep[j] = tmp;
        _rep[j]->set_heap_index(j);
    }
    bool _heapify_up(unsigned i) {
        unsigned original_i = i;
        while (i > 1 && _less(_rep[i], _rep[_parent(i)])) {
            _exchange(i, _parent(i));
            i = _parent(i);
        }
        return i != original_i;
    }
    void _heapify_down(unsigned i) {
        unsigned largest = i;
        unsigned l = _left(i);
        if (l <= _size && _less(_rep[l], _rep[i]))
            largest = l;
        unsigned r = _right(i);
        if (r <= _size && _less(_rep[r], _rep[largest]))
            largest = r;
        if (largest != i) {
            _exchange(i, largest);
            _heapify_down(largest); // tail recursion
        }
    }
};

#endif // __Heap_h__
