#include <iostream>
#include <vector>
#include <deque>
#include <algorithm>
#include <cmath>
#include "sq.h"

using std::vector;
using std::deque;
using std::sort;

class UniformTrainer : public ScalarQuantizerTrainer {
 public:
    UniformTrainer(unsigned nBits, float trimLow, float trimHigh);
    virtual ~UniformTrainer();
    virtual void train(vector<float>& d, vector<float>& r);
protected:
    unsigned nBits;
    float trimLow, trimHigh;
};

UniformTrainer::UniformTrainer(unsigned nBits, float trimLow, float trimHigh)
{
    this->nBits = nBits;
    this->trimLow = trimLow;
    this->trimHigh = trimHigh;
}

UniformTrainer::~UniformTrainer()
{
}

static void
updateR(vector<float>& r, const vector<float>& d, const deque<float>& data, unsigned n)
{
    float sum = 0;
    int count = 0;
    unsigned i = 0;
    for (deque<float>::const_iterator j = data.begin(); j != data.end(); ++j) {
        float e = *j;
        if (i == n - 1 || e <= d[i]) {
            sum += e;
            ++count;
        } else {
            r[i] = sum / count;
            sum = e;
            count = 1;
            ++i;
        }
    }
    r[i] = sum / count;
}

void
UniformTrainer::train(vector<float>& d, vector<float>& r)
{
    unsigned n = 1 << nBits;

    sort(data.begin(), data.end());

    unsigned ld = unsigned(data.size() * trimLow);
    unsigned hd = unsigned(data.size() * trimHigh);

    float l = data[ld].first;
    float h = data[data.size() - 1 - hd].first;

    vector<float>(n - 1).swap(d);
    vector<float>(n).swap(r);

#if 1

    //l = -2;
    //h = 2 - powf(2,-16);

#if 1
    // symmetric about zero
    float lh = std::min(fabsf(l), fabsf(h));
    lh = 2.4; // hand hack
    l = -lh;
    h = +lh;
#endif

    for (unsigned i = 0; i < n; i++) {
        r[i] = l + i*(h - l)/(n - 1);
    }
    for (unsigned i = 0; i < n - 1; i++) {
        d[i] = (r[i] + r[i + 1])/2;
    }
#else
    for (unsigned i = 1; i < n; i++) {
        d[i - 1] = l + (i*(h - l))/n;
    }
    updateR(r, d, data, n);
#endif

#if 0
    using std::cerr;
    cerr << "\nl=" << l << " h=" << h << '\n';
    cerr << "scale=" << ((h - l)/(n - 1)) << '\n';
    cerr << "l  -> " << sq_quantize(l, &d[0], n) << '\n';
    cerr << "0- -> " << sq_quantize(-1e-6, &d[0], n) << '\n';
    cerr << "0  -> " << sq_quantize(0, &d[0], n) << '\n';
    cerr << "0+ -> " << sq_quantize(+1e-6, &d[0], n) << '\n';
    cerr << "h  -> " << sq_quantize(h, &d[0], n) << '\n';

    cerr << "d:";
    for (unsigned i = 1; i < n; i++) {
        cerr << ' ' << d[i - 1];
    }
    cerr << "\nr:";
    for (unsigned i = 0; i < n; i++) {
        cerr << ' ' << r[i];
    }
    cerr << '\n';
#endif

    deque<FloatFloat>().swap(data); // clear and deallocate data
}

void* sq_trainer_uniform_new(unsigned nBits, float trim_fraction_low, float trim_fraction_high)
{
    return new UniformTrainer(nBits, trim_fraction_low, trim_fraction_high);
}
