#ifndef __min_max_heap_x__
#define __min_max_heap_x__

#ifdef __cplusplus
extern "C" {
#endif

typedef struct {
    /* intentionally empty */
} MinMaxHeap;

typedef int  (*MinMaxHeapCompareFunc)(const void* x, const void* y, void* data);
typedef void (*MinMaxHeapIndexFunc)(void* x, unsigned index, void* data);

MinMaxHeap* min_max_heap_new(unsigned int element_size,
                             unsigned int initial_capacity,
                             MinMaxHeapCompareFunc compare_func,
                             MinMaxHeapIndexFunc index_func,
                             void* compare_data);
void         min_max_heap_delete(MinMaxHeap* h);
void         min_max_heap_clear(MinMaxHeap* h);
unsigned int min_max_heap_size(MinMaxHeap* h);
const void*  min_max_heap_data(MinMaxHeap* h);    
void         min_max_heap_push(MinMaxHeap* h, const void* source);
int          min_max_heap_peek_min(MinMaxHeap* h, void* destination);
int          min_max_heap_pop_min(MinMaxHeap* h, void* destination);
int          min_max_heap_peek_max(MinMaxHeap* h, void* destination);
int          min_max_heap_pop_max(MinMaxHeap* h, void* destination);

/* min_max_heap_preadjust(h) must be called before adjusting a value
   and then calling min_max_heap_adjust(h, i). */
void         min_max_heap_preadjust(MinMaxHeap* h);
void         min_max_heap_adjust(MinMaxHeap* h, unsigned index);

unsigned int min_max_heap_verify(MinMaxHeap* h);

#ifdef __cplusplus
}
#endif

#endif
