// -*- C++ -*-

#ifndef __sq_h__
#define __sq_h__

#ifdef __cplusplus

#include <vector>
#include <deque>

class ScalarQuantizerTrainer {
public:
    virtual ~ScalarQuantizerTrainer();
    virtual void incorporate(float x, float weight);
    virtual void incorporate(const float* begin, const float* end, float weight);
    virtual void train(std::vector<float>& d, std::vector<float>& r) = 0;
protected:
    typedef std::pair<float,float> FloatFloat;
    std::deque<FloatFloat> data;
};

extern "C" {

#endif // __cplusplus

void* sq_trainer_lloyd_max_new(unsigned bits);
void* sq_trainer_quantile_new(unsigned bits);
void* sq_trainer_uniform_new(unsigned bits, float trim_fraction_low, float trim_fraction_high);

void  sq_trainer_incorporate(void* trainer, float x, float weight);
void  sq_trainer_incorporate_sequence(void* trainer, const float* begin, const float* end, float weight);
void  sq_trainer_train(void* trainer, float** d, float** r);
void  sq_trainer_delete(void* trainer);

unsigned sq_quantize(float x, const float* d, unsigned n);

#ifdef __cplusplus

}

#endif // __cplusplus

#endif // __sq_h__
