#include <iostream>
#include <set>
#include "fst.h"

using namespace std;

FstTool* fst_tool_list;

static void
usage()
{
    set<string> subcommands;
    for (FstTool* i = fst_tool_list; i != NULL; i = i->next()) {
        subcommands.insert(i->name());
    }

    cerr <<
        "usage: fst <subcommand> [options] [args]\n"
        "\n"
        "Available subcommands:\n";
    for (set<string>::iterator i = subcommands.begin(); i != subcommands.end(); ++i) {
        cerr << "   " << *i << '\n';
    }
    cerr << '\n';
}

static FstTool*
lookup_tool(const string& name)
{
    for (FstTool* i = fst_tool_list; i != NULL; i = i->next()) {
        if (i->name() == name) {
            return i;
        }
    }
    return NULL;
}

static string
extract_subcommand(string s)
{
    string::size_type i;

    i = s.find_last_of("/\\");
    if (i != string::npos) {
        s = s.substr(i + 1);
    }

    i = s.rfind("_debug");
    if (i == s.length() - 6) {
        s = s.substr(0, s.length() - 6);
    }

    i = s.rfind("_profile");
    if (i == s.length() - 8) {
        s = s.substr(0, s.length() - 8);
    }

    return (s.find("fst_") == 0) ? s.substr(4) : "";
}

int
main(int argc, char** argv)
{
    string subcommand = extract_subcommand(argv[0]);
    if (subcommand.length() > 0) {
        FstTool* tool = lookup_tool(subcommand);
        if (tool) {
            return tool->run(argc, argv);
        } else {
            usage();
            return 1;
        }
    } else {
        if (argc > 1) {
            if (string(argv[1]) == string("help")) {
                if (argc == 3) {
                    FstTool* tool = lookup_tool(argv[2]);
                    if (tool) {
                        tool->help();
                        return 0;
                    }
                } else {
                    usage();
                    return 0;
                }
            } else {
                FstTool* tool = lookup_tool(argv[1]);
                if (tool) {
                    return tool->run(argc - 1, argv + 1);
                }
            }
        }
        usage();
        return 1;
    }
}
