////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2005, Massachusetts Institute of Technology              //
// All rights reserved.                                                   //
//                                                                        //
// Redistribution and use in source and binary forms, with or without     //
// modification, are permitted provided that the following conditions are //
// met:                                                                   //
//                                                                        //
//     * Redistributions of source code must retain the above copyright   //
//       notice, this list of conditions and the following disclaimer.    //
//                                                                        //
//     * Redistributions in binary form must reproduce the above          //
//       copyright notice, this list of conditions and the following      //
//       disclaimer in the documentation and/or other materials provided  //
//       with the distribution.                                           //
//                                                                        //
//     * Neither the name of the Massachusetts Institute of Technology    //
//       nor the names of its contributors may be used to endorse or      //
//       promote products derived from this software without specific     //
//       prior written permission.                                        //
//                                                                        //
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS    //
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT      //
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR  //
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT   //
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,  //
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT       //
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,  //
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY  //
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT    //
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE  //
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.   //
////////////////////////////////////////////////////////////////////////////

#include <iostream>
#include "fst.h"

using namespace std;

static void
usage()
{
    cerr << "usage: fst enumerate_paths in.fst\n\n";
}

typedef std::vector<int> IntVector;

static void
dfs(IntVector& inputs, int output, const FSTGeneric& fst, int p)
{
    if (output != 0 && fst->is_final(p)) {
        cout << FSTstring_map[output] << " :";
        for (IntVector::iterator i = inputs.begin(); i != inputs.end(); i++) {
            cout << ' ' << FSTstring_map[*i];
        }
        cout << endl;
    }
    for (FSTArcIter a = fst->arcs_out(p); a.valid(); a++) {
        if (a->next != p) {
            assert(output == 0 || a->output == 0);
            if (a->input) {
                inputs.push_back(a->input);
            }
            dfs(inputs, output ? output : a->output, fst, a->next);
            if (a->input) {
                inputs.pop_back();
            }
        }
    }
}

static int
fst_enumerate_paths(int argc, char** argv)
{
    if (argc != 2) {
        usage();
        return 1;
    }

    FSTGeneric fst = FSTLoad(argv[1]);
    IntVector inputs;
    dfs(inputs, 0, fst, fst->initial());

    return 0;
}

// Register as subcommand.
//
static FstTool enumerate_paths_tool("enumerate_paths", fst_enumerate_paths);

// for Emacs...
// Local Variables:
// mode: C++
// comment-column: 48
// fill-column: 80
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
