////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2005, Massachusetts Institute of Technology              //
// All rights reserved.                                                   //
//                                                                        //
// Redistribution and use in source and binary forms, with or without     //
// modification, are permitted provided that the following conditions are //
// met:                                                                   //
//                                                                        //
//     * Redistributions of source code must retain the above copyright   //
//       notice, this list of conditions and the following disclaimer.    //
//                                                                        //
//     * Redistributions in binary form must reproduce the above          //
//       copyright notice, this list of conditions and the following      //
//       disclaimer in the documentation and/or other materials provided  //
//       with the distribution.                                           //
//                                                                        //
//     * Neither the name of the Massachusetts Institute of Technology    //
//       nor the names of its contributors may be used to endorse or      //
//       promote products derived from this software without specific     //
//       prior written permission.                                        //
//                                                                        //
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS    //
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT      //
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR  //
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT   //
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,  //
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT       //
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,  //
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY  //
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT    //
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE  //
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.   //
////////////////////////////////////////////////////////////////////////////

/******************************************************************************
=head1 NAME

fst_from_string - convert string of arguments into FST

=head1 SYNOPSIS

B<fst_from_string> I<fst-out> I<symbol>...

=head1 DESCRIPTION

B<fst_from_string> is a very simple tool to quickly create an FST from
command-line arguments.  Actually, it produces an FSA because the input and
output symbols on each arc are identical.

For example,

    % fst_from_string out.fst a b c d

will produce an FST file C<out.fst> with the sequence of arcs C<a:a b:b c:c
d:d>.

The specification of the output FST I<fst-out> can be a filename or "-"
indicating standard output.

=head1 OPTIONS

None.

=head1 AUTHOR

I. Lee Hetherington <ilh@mit.edu>

=cut
******************************************************************************/

#include <iostream>
#include "fst.h"

using namespace std;

static void
usage()
{
    cerr << "usage: fst from_string out.fst words*\n\n";
}

static int
fst_from_string(int argc, char** argv)
{
    if (argc < 2) {
    usage();
        return 1;
    }

    const char* out_file  = argv[1];
    argc -= 2;
    argv += 2;

    FSTGeneric out = FSTBasic();

    int p = out->add_state();
    out->set_initial(p);

    if (argc > 0) {
        for (int i = 0; i < argc; i++) {
            int label = FSTstring_map[argv[i]];
            p = out->add_state();
            out->add_arc(p - 1, p, label, label);
        }
    } else {
        string item;
        while (cin >> item) {
            int label = FSTstring_map[item];
            p = out->add_state();
            out->add_arc(p - 1, p, label, label);
        }
    }

    out->set_final(p);

    out->dump_ascii(out_file);

    return 0;
}

// Register as subcommand.
//
static FstTool from_string_tool("from_string", fst_from_string);

// for Emacs...
// Local Variables:
// mode: C++
// comment-column: 48
// fill-column: 80
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:


