////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2005, Massachusetts Institute of Technology              //
// All rights reserved.                                                   //
//                                                                        //
// Redistribution and use in source and binary forms, with or without     //
// modification, are permitted provided that the following conditions are //
// met:                                                                   //
//                                                                        //
//     * Redistributions of source code must retain the above copyright   //
//       notice, this list of conditions and the following disclaimer.    //
//                                                                        //
//     * Redistributions in binary form must reproduce the above          //
//       copyright notice, this list of conditions and the following      //
//       disclaimer in the documentation and/or other materials provided  //
//       with the distribution.                                           //
//                                                                        //
//     * Neither the name of the Massachusetts Institute of Technology    //
//       nor the names of its contributors may be used to endorse or      //
//       promote products derived from this software without specific     //
//       prior written permission.                                        //
//                                                                        //
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS    //
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT      //
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR  //
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT   //
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,  //
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT       //
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,  //
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY  //
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT    //
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE  //
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.   //
////////////////////////////////////////////////////////////////////////////

#include <cstring>
#include <iostream>
#include <fstream>
#include "fst.h"

using namespace std;

static void
usage()
{
    cerr <<
        "usage: fst ignore [-i] [-m] [-f] [-a] in.labels in.fst out.fst\n"
        "\n"
        "-i     initial\n"
        "-m     middle\n"
        "-f     final\n"
        "-a     absorb\n"
        "\n";
}

static int
fst_ignore(int argc, char** argv)
{
    if (argc < 0) {
        usage();
        return 1;
    }

    char c;
    bool error = false;
    bool initial = true;
    bool middle = true;
    bool final = true;
    bool absorb = false;

    while ((c = getopt(argc, argv, "imfa")) != EOF) {
      switch (c) {
      case 'i':
    initial = false;
    break;
      case 'm':
    middle = false;
    break;
      case 'f':
    final = false;
    break;
      case 'a':
    absorb = true;
    break;
      default:
    error = true;
    break;
      }
    }
    argc -= optind;
    argv += optind;
    
    if (error || argc != 3) {
      usage();
      return 1;
    }

    const char* labels_filename  = argv[0];
    const char* in_file  = argv[1];
    const char* out_file = argv[2];

    ifstream is(labels_filename);
    if (is.bad()) {
      cerr << "error reading file `" << labels_filename << "'" << endl;
      return 1;
    }

    FSTGeneric x;
    if(strcmp(in_file, "_") && strcmp(in_file, "None"))
      x = FSTMutableCopy(FSTLoad(in_file));
    else {
      x = FSTBasic();
      x->add_states(1);
      x->set_initial(0);
      x->set_final(0);
    }
    int initial_state = x->initial();

    // Assume ignored labels are disjoint to input alphabet
    typedef fst::vector<int> IntVector;
    IntVector labels;
    string line;
    while (!getline(is, line).fail())
      labels.push_back(FSTstring_map[line]);
    for(IntVector::const_iterator i=labels.begin();i!=labels.end();i++)
      for(FSTStateIter si=x->states();si.valid();si++)
    if((initial && *si == initial_state) ||
       (final && x->is_final(*si)) ||
       (middle && *si != initial_state && !x->is_final(*si))){
        if(!absorb)
            x->add_arc(*si, *si, *i, *i);
        else
            x->add_arc(*si, *si, *i);
    }

    x->dump_ascii(out_file);

    return 0;
}

// Register as subcommand.
//
static FstTool ignore_tool("ignore", fst_ignore);

// for Emacs...
// Local Variables:
// mode: C++
// comment-column: 48
// fill-column: 80
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
