////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2005, Massachusetts Institute of Technology              //
// All rights reserved.                                                   //
//                                                                        //
// Redistribution and use in source and binary forms, with or without     //
// modification, are permitted provided that the following conditions are //
// met:                                                                   //
//                                                                        //
//     * Redistributions of source code must retain the above copyright   //
//       notice, this list of conditions and the following disclaimer.    //
//                                                                        //
//     * Redistributions in binary form must reproduce the above          //
//       copyright notice, this list of conditions and the following      //
//       disclaimer in the documentation and/or other materials provided  //
//       with the distribution.                                           //
//                                                                        //
//     * Neither the name of the Massachusetts Institute of Technology    //
//       nor the names of its contributors may be used to endorse or      //
//       promote products derived from this software without specific     //
//       prior written permission.                                        //
//                                                                        //
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS    //
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT      //
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR  //
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT   //
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,  //
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT       //
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,  //
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY  //
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT    //
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE  //
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.   //
////////////////////////////////////////////////////////////////////////////

#include <iostream>
#include "fst.h"

using namespace std;

static void
usage()
{
    cerr <<
        "usage: fst mmap [-o] (in.fst out.fst)+\n" 
        "\n"
        "       Convert to mmap format for fast, on-demand loading.\n"
        "\n";
}

static int
fst_mmap(int argc, char** argv)
{
    if (argc < 0) {
        usage();
        return 1;
    }

    bool output_flag = false;
    bool error = false;
    char c;
    while ((c = getopt(argc, argv, "o")) != EOF) {
        switch (c) {
        case 'o':
            output_flag = true;
            break;
        default:
            error = true;
            break;
        }
    }
    argc -= optind;
    argv += optind;

    if (argc % 2 != 0) {
    usage();
        return 1;
    }

    FSTWarn("fst mmap is experimental and its file format is in flux.");

    // Read in FSTs
    int num_pairs = argc / 2;
    FSTGeneric *fsts = new FSTGeneric[num_pairs];
    for(int i=0;i<num_pairs;i++)
        fsts[i] = FSTLoad(argv[2*i]);

    // Write out FSTs
    for(int i=0;i<num_pairs;i++)
        fsts[i]->dump_mmap(argv[2*i + 1], output_flag ? FSTOutput : FSTInput);
    delete[] fsts;

    return 0;
}

// Register as subcommand.
//
static FstTool mmap_tool("mmap", fst_mmap);

// for Emacs...
// Local Variables:
// mode: C++
// comment-column: 48
// fill-column: 80
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
