////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2005, Massachusetts Institute of Technology              //
// All rights reserved.                                                   //
//                                                                        //
// Redistribution and use in source and binary forms, with or without     //
// modification, are permitted provided that the following conditions are //
// met:                                                                   //
//                                                                        //
//     * Redistributions of source code must retain the above copyright   //
//       notice, this list of conditions and the following disclaimer.    //
//                                                                        //
//     * Redistributions in binary form must reproduce the above          //
//       copyright notice, this list of conditions and the following      //
//       disclaimer in the documentation and/or other materials provided  //
//       with the distribution.                                           //
//                                                                        //
//     * Neither the name of the Massachusetts Institute of Technology    //
//       nor the names of its contributors may be used to endorse or      //
//       promote products derived from this software without specific     //
//       prior written permission.                                        //
//                                                                        //
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS    //
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT      //
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR  //
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT   //
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,  //
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT       //
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,  //
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY  //
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT    //
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE  //
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.   //
////////////////////////////////////////////////////////////////////////////

#include <iostream>
#include "fst.h"

#ifdef _MSC_VER
#  pragma warning(disable: 4065)
#endif

using namespace std;

static void
usage()
{
    cerr <<
        "usage: fst ngram_difference a.fst b.fst out.fst\n"
        "\n"
        "       Compute incremental or difference n-gram between a.fst and b.fst,\n"
        "       where both a.fst and b.fst are FSTNgram2 files.\n"
        "\n";
}

static int
fst_ngram_difference(int argc, char** argv)
{
    if (argc < 0) {
        usage();
        return 1;
    }

    bool error = false;

    char c;
    while ((c = getopt(argc, argv, "")) != EOF) {
        switch (c) {
        default:
            error = true;
            break;
        }
    }
    argc -= optind;
    argv += optind;
    if (error || argc != 3) {
        usage();
        return 1;
    }

    const char* a_file   = *argv++;
    const char* b_file   = *argv++;
    const char* out_file = *argv++;

    FSTGeneric a = FSTLoad(a_file);
    FSTGeneric b = FSTLoad(b_file);

    FSTNgramSubtract(a, b);

    a->dump_binary(out_file);

    return 0;
}

// Register as subcommand.
//
static FstTool ngram_difference_tool("ngram_difference", fst_ngram_difference);

// for Emacs...
// Local Variables:
// mode: C++
// comment-column: 48
// fill-column: 80
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
