# -*- Makefile -*-

#   (c) Copyright 2003
#       Massachusetts Institute of Technology
#       Spoken Language Systems Group
#       All Rights Reserved
#

# liba.make -- build .a library file
#
# typical usage:
#
# FST_HOME = ../..
# SOURCES = foo.c bar.cpp
# TARGET = $(FST_HOME)/lib/$(ARCHOS)/libfoo.a
# include $(FST_HOME)/templates/liba.make

ifndef __A_MAKE__
__A_MAKE__=1

default:	normal man
all:		normal debug profile man

include $(FST_HOME)/templates/objects.make

TARGET_N	= $(TARGET)
TARGET_D	= $(TARGET:%.a=%_debug.a)
TARGET_P	= $(TARGET:%.a=%_profile.a)
TARGET_I	= $(TARGET:%.a=%_insure.a)

FST_LIBS_N	= $(FST_HOME)/lib/$(ARCHOS)/libfst.a
FST_LIBS_D	= $(FST_LIBS_N:%.a=%_debug.a)
FST_LIBS_P	= $(FST_LIBS_N:%.a=%_profile.a)
FST_LIBS_I	= $(FST_LIBS_I:%.a=%_insure.a)

DEPEND_FILES	= $(OBJ_N:%.o=%.d) $(OBJ_D:%.o=%.d) $(OBJ_P:%.o=%.d) $(OBJ_I:%.o=%.d)

normal:		$(TARGET_N)
debug:		$(TARGET_D)
profile:	$(TARGET_P)
insure:		$(TARGET_I)

$(TARGET_N):	$(OBJ_N) $(OBJECTS)
		-@$(MKDIR) $(dir $@)
		@echo $(AR) $@ ...
		@$(AR) $@ $?
		$(RANLIB) $@

$(TARGET_D):	$(OBJ_D) $(OBJECTS)
		-@$(MKDIR) $(dir $@)
		@echo $(AR) $@ ...
		@$(AR) $@ $?
		$(RANLIB) $@

$(TARGET_P):	$(OBJ_P) $(OBJECTS)
		-@$(MKDIR) $(dir $@)
		@echo $(AR) $@ ...
		@$(AR) $@ $?
		$(RANLIB) $@

$(TARGET_I):	$(OBJ_I) $(OBJECTS)
		-@$(MKDIR) $(dir $@)
		@echo $(AR) $@ ...
		@$(AR) $@ $?
		$(RANLIB) $@

MANDIR = $(FST_HOME)/man

MAN3_TARGETS = $(patsubst %.3.pod,$(MANDIR)/man3/%.3,$(filter %.3.pod,$(MAN_SOURCES)))
MAN5_TARGETS = $(patsubst %.5.pod,$(MANDIR)/man5/%.5,$(filter %.5.pod,$(MAN_SOURCES)))

$(MANDIR)/man3/%.3:	%.3.pod
	-@$(MKDIR) $(dir $@)
	$(POD2MAN) -r $(POD_PACKAGE) -s 3 -c $(POD_CENTER) -d $(POD_VERSION) $< | sed 's/\.3 3/ 3/' > $@

$(MANDIR)/man5/%.5:	%.5.pod
	-@$(MKDIR) $(dir $@)
	$(POD2MAN) -r $(POD_PACKAGE) -s 5 -c $(POD_CENTER) -d $(POD_VERSION) $< | sed 's/\.5 5/ 5/' > $@

man:		$(MAN3_TARGETS) $(MAN5_TARGETS)

clean:		clean_normal clean_debug clean_profile clean_insure clean_extra clean_man

clean_normal:
		rm -f $(OBJ_N) $(OBJ_N:%.o=%.d) $(TARGET_N)
clean_debug:
		rm -f $(OBJ_D) $(OBJ_D:%.o=%.d) $(TARGET_D)
clean_profile:
		rm -f $(OBJ_P) $(OBJ_P:%.o=%.d) $(TARGET_P)
clean_insure:
		rm -f $(OBJ_I) $(OBJ_I:%.o=%.d) $(TARGET_I)
clean_extra:
		rm -f $(EXTRA_CLEAN)
clean_man:
		rm -f $(MAN3_TARGETS) $(MAN5_TARGETS)

include $(wildcard $(DEPEND_FILES))

.PRECIOUS:	$(TARGET_N) $(TARGET_D) $(TARGET_P)

.PHONY:		default \
		all \
		normal \
		debug \
		profile \
		insure \
		man \
		clean \
		clean_normal \
		clean_debug \
		clean_profile \
		clean_insure \
		clean_extra \
		clean_man \

endif # ifdef __A_MAKE__




