# -*- Makefile -*-

#   (c) Copyright 2005
#       Massachusetts Institute of Technology
#       Spoken Language Systems Group
#       All Rights Reserved
#

# program.make -- build executable program
#
# typical usage:
#
# FST_HOME = ../..
# SOURCES = foo.c bar.cpp
# TARGET = $(FST_HOME)/bin/foo
# include $(FST_HOME)/templates/program.make

ifndef __EXEC_MAKE__
__EXEC_MAKE__=1

default:	normal
all:		normal debug

include $(FST_HOME)/templates/objects.make

TARGET_DIR	= $(dir $(TARGET))
TARGET_NOTDIR	= $(notdir $(TARGET))

TARGET_N	= $(dir $(TARGET))$(ARCHOS)/$(notdir $(TARGET))
TARGET_D	= $(TARGET_N:%=%_debug)
TARGET_P	= $(TARGET_N:%=%_profile)
TARGET_I	= $(TARGET_N:%=%_insure)

ALL_TARGETS	= $(TARGET) $(TARGET)_debug $(TARGET)_profile $(TARGET)_insure

FAT_BINARY	= $(TARGET_DIR)fat_binary
FAT_BINARY_SRC	= $(FST_HOME)/templates/fat_binary.csh

FST_LIBS_N	= $(FST_HOME)/lib/$(ARCHOS)/libfst.a

FST_LIBS_D	= $(FST_LIBS_N:%.a=%_debug.a)
FST_LIBS_P	= $(FST_LIBS_N:%.a=%_profile.a)
FST_LIBS_I	= $(FST_LIBS_N:%.a=%_insure.a)

DEPEND_FILES	= $(OBJ_N:%.o=%.d) $(OBJ_D:%.o=%.d) $(OBJ_P:%.o=%.d) $(OBJ_I:%.o=%.d)

normal:		$(TARGET) $(TARGET_N)
debug:		$(TARGET)_debug $(TARGET_D)
profile:	$(TARGET)_profile $(TARGET_P)
insure:		$(TARGET)_insure $(TARGET_I)

ifeq ($(CC),cl)
    OFLAG := /Fe
else
    OFLAG := -o
endif

$(ALL_TARGETS):	$(FAT_BINARY)
		rm -f $@
		ln -s $(notdir $(FAT_BINARY)) $@
		touch $@

$(FAT_BINARY):	$(FAT_BINARY_SRC)
		-@$(MKDIR) $(dir $@)
		rm -f $@
		cp $(FAT_BINARY_SRC) $@
		chmod +x $@

$(TARGET_N):	$(OBJ_N) $(FST_LIBS_N)
		-@$(MKDIR) $(dir $@)
		$(CXX) $(CFLAGS_N) $(CFLAGS) $(LDFLAGS) $(OFLAG) $@.tmp $(OBJ_N) $(FST_LIBS_N) $(LIBS)
		$(STRIP) $@.tmp
		mv $@.tmp $@

$(TARGET_D):	$(OBJ_D) $(FST_LIBS_D)
		-@$(MKDIR) $(dir $@)
		$(CXX) $(CFLAGS_D) $(CFLAGS) $(LDFLAGS) $(OFLAG) $@.tmp $(OBJ_D) $(FST_LIBS_D) $(LIBS)
		mv $@.tmp $@

$(TARGET_P):	$(OBJ_P) $(FST_LIBS_P)
		-@$(MKDIR) $(dir $@)
		$(CXX) $(CFLAGS_P) $(CFLAGS) $(LDFLAGS) $(OFLAG) $@.tmp $(OBJ_P) $(FST_LIBS_P) $(LIBS)
		mv $@.tmp $@

$(TARGET_I):	$(OBJ_I) $(FST_LIBS_I)
		-@$(MKDIR) $(dir $@)
		$(INSURE_CXX) $(CFLAGS_I) $(CFLAGS) $(LDFLAGS) $(OFLAG) $@.tmp $(OBJ_I) $(FST_LIBS_I) $(LIBS)
		mv $@.tmp $@

clean:		clean_normal clean_debug clean_profile clean_insure clean_man

clean_normal:
		rm -f $(OBJ_N) $(OBJ_N:%.o=%.d) $(TARGET_N)
clean_debug:
		rm -f $(OBJ_D) $(OBJ_D:%.o=%.d) $(TARGET_D)
clean_profile:
		rm -f $(OBJ_P) $(OBJ_P:%.o=%.d) $(TARGET_P)
clean_insure:
		rm -f $(OBJ_I) $(OBJ_I:%.o=%.d) $(TARGET_I)

depend:

include $(wildcard $(DEPEND_FILES))

.PHONY:		default \
		all \
		normal \
		debug \
		profile \
		insure \
		clean \
		clean_normal \
		clean_debug \
		clean_profile \
		clean_insure \

endif # ifdef __EXEC_MAKE__







