# -*- Makefile -*-

ifndef __SYSDEP_MAKE__
__SYSDEP_MAKE__ = 1

ARCH := $(shell $(FST_HOME)/templates/guess-arch.csh)
OS := $(shell $(FST_HOME)/templates/guess-os.csh)

# Normalize ARCH.
#
ifeq ($(ARCH),i86pc)
    ARCH := x86
endif
ifeq ($(ARCH),x86_64)
    ARCH := x86_64
endif
ifeq ($(ARCH),ia64)
    ARCH := ia64
endif
ifeq ($(ARCH),sun4)
    ARCH := sparc
endif

# Normalize OS.
#
ifeq ($(OS),Windows_NT)
    OS := winnt
endif

ARCHOS 			:= $(ARCH)-$(OS)

CPPFLAGS		+= -D__FST__ -D__$(ARCH)__ -D__$(OS)__ -D_NOTHREADS
CPPFLAGS_D		+= -DDEBUG

# Defaults across all platforms.  You can override them for specific
# platforms below.  Most of these assume gcc/g++.
#
USING_GCC               = 1
CC			= gcc
CXX			= g++
AS			= as
CFLAGS			+= -fno-strict-aliasing -fno-exceptions -Wall
CFLAGS_I		= $(CFLAGS_D)
AR			= ar crsv
BUILD_LIB_A		= $(AR)
RANLIB			= touch
MKDIR			= mkdir -p
PERL			= perl
STRIP			= strip

INSURE_FLAGS		= -Zsl -D__INSURE__ -D__USE_MALLOC
INSURE_CC		= insure -Zoi 'compiler gcc' $(INSURE_FLAGS)
INSURE_CXX		= insure -Zoi 'compiler g++' $(INSURE_FLAGS)

# ARCH-specific settings.
#
ifeq ($(ARCH),x86)
    SSE_FLAG		:= $(shell csh -f $(FST_HOME)/templates/sse-flag.csh $(CC))
    CFLAGS		+= -march=i686 $(SSE_FLAG)
    CFLAGS_N		+= -O3 -fomit-frame-pointer
    CFLAGS_D		+= -g
    CFLAGS_P		+= -pg -O3
endif

ifeq ($(ARCH),x86_64)
    CFLAGS		+= -march=nocona -m64 -mpreferred-stack-boundary=4
    CFLAGS_N		+= -O2 -fomit-frame-pointer
    CFLAGS_D		+= -g
    CFLAGS_P		+= -pg -O2
endif

ifeq ($(ARCH),ia64)
    CFLAGS		+= -mlittle-endian
    CFLAGS_N		+= -O3
    CFLAGS_D		+= -g
    CFLAGS_P		+= -pg -O3
endif

ifeq ($(ARCH),sparc)
    CFLAGS		+= -msupersparc
    CFLAGS_N		+= -O4
    CFLAGS_D		+= -g
    CFLAGS_P		+= -pg -O4
    CPPFLAGS		+= -DADIO_COMP_SUN
endif

ifeq ($(ARCH),alpha)
    CFLAGS_N		+= -O4
    CFLAGS_D		+= -g
    CFLAGS_P		+= -pg -O4
    CPPFLAGS		+= -D_THREAD_SAFE
endif

# OS-specific settings.
#
ifeq ($(OS),linux)
    CPPFLAGS		+= -D__USE_ISOC9X
    LIBS		+= -lm
    BUILD_LIB_SO	= $(CXX) -shared
endif

ifeq ($(OS),osx)
    CPPFLAGS            += -D__USE_ISOC9X
    LIBS                += -lm
    BUILD_LIB_SO        = $(CXX) -shared
endif

ifeq ($(OS),solaris)
    LIBS		+= -lrpcsvc -lkvm -lelf -laudio -lsocket -lnsl -lintl \
			   /usr/lib/libdl.so /usr/lib/libposix4.so -lm
    BUILD_LIB_SO	= $(CXX) -G
endif

# ARCHOS-specific settings.
#

ifeq ($(ARCHOS),x86-winnt)
    CYGWIN := $(shell uname -o)
    ifeq ($(CYGWIN),Cygwin)
        CPPFLAGS	+= -D__USE_ISOC9X -D__cygwin__
        LIBS		+= -lm
        BUILD_LIB_SO	= $(CXX) -shared
    else 
        USING_GCC	= 0
        CC		= cl
        CXX		= cl
        CFLAGS		= /GX /GR
        CFLAGS_N	= 
        CFLAGS_D	= 
        CFLAGS_P	= 
        LIBS		= 
    endif
endif

ifeq ($(ARCHOS),x86-solaris)
    CPPFLAGS		+=  -DADIO_COMP_I86PC_SOLARIS
endif

#
# POD documentation settings.
#
POD2MAN = pod2man
ifndef POD_PACKAGE
    POD_PACKAGE = "MIT FST Toolkit"
endif
ifndef POD_CENTER
    POD_CENTER = $(POD_PACKAGE)
endif
ifndef POD_VERSION
    POD_VERSION = "version 1.2"
endif
ifndef MANDIR
    MANDIR = $(FST_HOME)/man
endif

endif # __SYSDEP_MAKE__
